package ch.epfl.ppo;

import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;
import java.util.NoSuchElementException;

public class InputStreamToIterable implements Iterable<Byte> {
    private final InputStream inStream;

    public InputStreamToIterable(InputStream inStream) {
        this.inStream = inStream;
    }

    @Override
    public Iterator<Byte> iterator() {
        return new Iterator<Byte>() {
            byte next = 0;
            boolean isNextValid = false;

            @Override
            public boolean hasNext() {
                if (!isNextValid) {
                    try {
                        int nextAsInt = inStream.read();
                        if (nextAsInt != -1) {
                            isNextValid = true;
                            next = (byte)nextAsInt;
                        } else
                            inStream.close();
                    } catch (IOException e) {
                        assert !isNextValid;
                    }
                }
                return isNextValid;
            }

            @Override
            public Byte next() {
                if (!hasNext())
                    throw new NoSuchElementException();
                isNextValid = false;
                return next;
            }

            @Override
            public void remove() {
                throw new UnsupportedOperationException();
            }
        };
    }
}
